#!/usr/bin/env python
import numpy as np
import matplotlib.pyplot as plt
import importlib
import os
import sys
import seaborn as sns
import pandas as pd
sys.path.append(os.getenv('HOME')+'/script')
import load_qe
import cal_band

hartree=27.211386245988
bohr=0.529177210903
cmap=sns.color_palette()
sns.set(style="ticks")

def read_eig(path):
    bandinfo=cal_band.band_data(path)
    eigenvalues=bandinfo['eigenvalues']
    fermi=eigenvalues[0,3]
    eigenvalues-=fermi
    return eigenvalues

fig,ax = plt.subplots(figsize=(3.1,5))
nb=8
ht=[]
wa=[]
wh=[]
x=np.array([3,4,5,6,7,8,9,10])
for i in x:
    e0=read_eig('k'+str(i)+'/ht')
    e1=read_eig('k'+str(i)+'/wannier')
    e2=read_eig('k'+str(i)+'/nscf')
    e3=read_eig('k'+str(i)+'/tmp')
    ht.append(np.mean(abs(e2[:,:nb]-e0[:,:nb])))
    wa.append(np.mean(abs(e2[:,:nb]-e1[:,:nb])))
    wh.append(np.mean(abs(e2[:,:nb]-e3[:,:nb])))
x=np.power(x,3)
df1=pd.DataFrame(ht,index=x)
df2=pd.DataFrame(wa,index=x)
df3=pd.DataFrame(wh,index=x)
#sns.lineplot(data=df1,legend=False,dashes=False,palette={x: cmap[0] for x in df1.columns},lw=1.5)
sns.lineplot(data=df2,legend=False,dashes=False,palette={x: cmap[1] for x in df2.columns},lw=1.5)
sns.lineplot(data=df3,legend=False,dashes=False,palette={x: cmap[2] for x in df2.columns},lw=1.5)
plt.yscale("log")
plt.xscale("log")

l0,=ax.plot([],[],color=cmap[0],linewidth=1.5)
l1,=ax.plot([],[],color=cmap[1],linewidth=1.5)
l2,=ax.plot([],[],color=cmap[2],linewidth=1.5)
#legend1 = plt.legend((l0,l1),('HT','Wannier'),loc=(0.05,0.2),frameon=False)
legend1 = plt.legend((l1,l2),('Wan mu=10','Wan mu=15'),loc=(0.05,0.05),frameon=False)
ax.add_artist(legend1)
ax.set_ylabel('Error (eV)')
ax.set_xlabel('Number of SCF k-points')
plt.savefig('accuracy.pdf', bbox_inches='tight')
plt.savefig('accuracy.png', bbox_inches='tight')
plt.show()
