#!/usr/bin/env python
import numpy as np
import matplotlib.pyplot as plt
import importlib
import os
import sys
import seaborn as sns
import pandas as pd
sys.path.append(os.getenv('HOME')+'/script')
import load_qe
import cal_band

hartree=27.211386245988
bohr=0.529177210903
cmap=sns.color_palette()
sns.set(style="ticks")

def read_eig(path):
    bandinfo=cal_band.band_data(path)
    eigenvalues=bandinfo['eigenvalues']
    fermi=eigenvalues[0,3]
    eigenvalues-=fermi
    return eigenvalues

fig,ax = plt.subplots(figsize=(3.1,5))
nb=8
x=np.array([3,4,5,6,7,8,9,10])
path='p1'
ht=[]
wh=[]
wa=[]
ws=[]
for i in x:
    e0=read_eig(path+'/k'+str(i)+'/ht')
    e1=read_eig(path+'/k'+str(i)+'/wan_ht')
    e2=read_eig(path+'/k'+str(i)+'/nscf')
    e3=read_eig(path+'/k'+str(i)+'/wannier')
    ht.append(np.mean(abs(e2[:,:nb]-e0[:,:nb])))
    wh.append(np.mean(abs(e2[:,:nb]-e1[:,:nb])))
    wa.append(np.mean(abs(e2[:,:nb]-e3[:,:nb])))
nk=np.power(x,3)
df1=pd.DataFrame(ht,index=nk)
df2=pd.DataFrame(wh,index=nk)
df3=pd.DataFrame(wa,index=nk)
sns.lineplot(data=df1,legend=False,dashes=False,palette={x: cmap[0] for x in df1.columns},lw=1.5,marker='o',markersize=6)
#sns.lineplot(data=df2,legend=False,dashes=False,palette={x: cmap[0] for x in df1.columns},lw=1.5,marker='o',markersize=5)
sns.lineplot(data=df3,legend=False,dashes=False,palette={x: 'k' for x in df2.columns},lw=1.5,marker='s',markersize=6,linestyle='--')

path='p2'
ht=[]
for i in x:
    e0=read_eig(path+'/k'+str(i)+'/ht')
    e2=read_eig(path+'/k'+str(i)+'/nscf')
    ht.append(np.mean(abs(e2[:,:nb]-e0[:,:nb])))
df1=pd.DataFrame(ht,index=nk)
sns.lineplot(data=df1,legend=False,dashes=False,palette={x: cmap[1] for x in df1.columns},lw=1.5,marker='o',markersize=6)

path='p3'
ht=[]
wh=[]
for i in x:
    e0=read_eig(path+'/k'+str(i)+'/ht')
    e1=read_eig(path+'/k'+str(i)+'/wan_ht')
    e2=read_eig(path+'/k'+str(i)+'/nscf')
    ht.append(np.mean(abs(e2[:,:nb]-e0[:,:nb])))
    wh.append(np.mean(abs(e2[:,:nb]-e1[:,:nb])))
df1=pd.DataFrame(ht,index=nk)
df2=pd.DataFrame(wh,index=nk)
sns.lineplot(data=df1,legend=False,dashes=False,palette={x: cmap[2] for x in df1.columns},lw=1.5,marker='o',markersize=6)
sns.lineplot(data=df2,legend=False,dashes=False,palette={x: cmap[2] for x in df1.columns},lw=1.5,marker='s',markersize=6, linestyle='--',zorder=-1)

path='p4'
ht=[]
for i in x:
    e0=read_eig(path+'/k'+str(i)+'/ht')
    e2=read_eig(path+'/k'+str(i)+'/nscf')
    ht.append(np.mean(abs(e2[:,:nb]-e0[:,:nb])))
df1=pd.DataFrame(ht,index=nk)
sns.lineplot(data=df1,legend=False,dashes=False,palette={x: cmap[3] for x in df1.columns},lw=1.5,marker='o',markersize=6)

plt.yscale("log")
plt.xscale("log")

l0,=ax.plot([],[],color=cmap[0],linewidth=1.5,marker='o',markersize=4.5)
l1,=ax.plot([],[],color=cmap[1],linewidth=1.5,marker='o',markersize=4.5)
l2,=ax.plot([],[],color=cmap[2],linewidth=1.5,marker='o',markersize=4.5)
l3,=ax.plot([],[],color=cmap[3],linewidth=1.5,marker='o',markersize=4.5)
#l4,=ax.plot([],[],color=cmap[2],linewidth=1.5,marker='s',markersize=4.5,linestyle='--')
#l5,=ax.plot([],[],color='k',linewidth=1.5,marker='s',markersize=4.5,linestyle='--')
#legend1 = plt.legend((l5,l4,l0,l1,l2,l3),('WI','WI+trans,\nn=3','HT n=1','HT n=2','HT n=3','HT n=4'),loc='lower left',frameon=False)
legend1 = plt.legend((l0,l1,l2,l3),('HT n=1','HT n=2','HT n=3','HT n=4'),loc='lower left',frameon=False)
ax.text(550,5e-2,'WI',horizontalalignment='center', verticalalignment='center')
ax.text(600,2.5e-3,'WI+trans\nn=3',horizontalalignment='center', verticalalignment='center')
#legend1 = plt.legend((l0,l4),('HT n=1','Wannier'),loc='lower left',frameon=False)
ax.add_artist(legend1)
ax.set_ylabel('Mean absolute error (eV)')
ax.set_xlabel('Number of SCF k-points ($N_k$)')
plt.savefig('Nk_MAE.pdf', bbox_inches='tight')
plt.show()
