#!/usr/bin/env python
import numpy as np
import matplotlib.pyplot as plt
import importlib
import os
import sys
import seaborn as sns
import pandas as pd
sys.path.append('../python_scripts')
import cal_band

hartree=27.211386245988
bohr=0.529177210903
cmap=sns.color_palette()
sns.set(style="ticks")

nb = 8
fig,ax = plt.subplots(figsize=(3.5,5))
bandinfo=cal_band.band_data('ht')
k=bandinfo['k']; eigenvalues=bandinfo['eigenvalues']; ktick=bandinfo['ktick'];
klabel=bandinfo['klabel']; software=bandinfo['software'];
fermi=np.amax(eigenvalues[:,3])
eigenvalues-=fermi
eigenvalues=eigenvalues[:,:nb]
df=pd.DataFrame(eigenvalues,index=k)
sns.lineplot(data=df,legend=False,dashes=False,palette={x: cmap[0] for x in df.columns},lw=1.5)
klabel[2]='X'
ax.set_xticks(ktick)
ax.set_xticklabels(klabel)
for i in ktick[:-1]: ax.axvline(x=i,color='k',linewidth=0.5)
ax.set_xlim(ktick[0],ktick[-3])
ax.set_ylabel('$E-E_f$ (eV)')

nb=8
data = np.loadtxt('wi.dat')
nk = len(data) // nb
x = data[:nk,0]
eig = data[:,1].reshape((nb,nk))
eig=eig.transpose()
fermi=np.amax(eig[:,3])
eig-=fermi
df=pd.DataFrame(eig,index=x)
sns.lineplot(data=df,legend=False,dashes=False,palette={x: cmap[1] for x in df.columns},lw=1.2,zorder=-1)

nb=8
data = np.loadtxt('wi_trans.dat')
nk = len(data) // nb
x = data[:nk,0]
eig = data[:,1].reshape((nb,nk))
eig=eig.transpose()
fermi=np.amax(eig[:,3])
eig-=fermi
df=pd.DataFrame(eig,index=x)
sns.lineplot(data=df,legend=False,dashes=False,palette={x: cmap[2] for x in df.columns},lw=1.2,zorder=-1)

nb=8
with open('inteqp.dat','r') as f:
    data=f.readlines()
eig=np.array([x.split() for x in data[2:]],dtype='float')[:,6].reshape([-1,202]).transpose()
fermi=np.amax(eig[:,3])
eig-=fermi
eig=eig[:,:nb]
df=pd.DataFrame(eig[::2,:],index=k[::2])
#sns.lineplot(data=df,legend=False,dashes=False,palette={x: cmap[1] for x in df.columns},lw=0.5)
sns.scatterplot(data=df,palette={x: cmap[3] for x in df.columns},markers=['o']*np.shape(df)[1],s=15,zorder=-20,legend=False)

l0,=ax.plot([],[],color=cmap[0],linewidth=1.5)
l1,=ax.plot([],[],color=cmap[1],linewidth=1.5)
l2,=ax.plot([],[],color=cmap[2],linewidth=1.5)
l3,=ax.plot([],[],'.',color=cmap[3],markersize=5)
legend1 = plt.legend((l0,l3),('HT','inteqp'),loc='upper left',frameon=False)
legend2 = plt.legend((l1,l2),('WI','WI+trans'),loc='upper right',frameon=False)
ax.add_artist(legend1)
ax.add_artist(legend2)
ax.set_xlabel('GW-silicon')
ax.set_ylabel('$E-E_f$ (eV)',labelpad=-10)
ax.set_ylim(-12.5,14.5)
plt.savefig('si2gw.pdf', bbox_inches='tight')
plt.show()
