#!/usr/bin/env python
import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit
import seaborn as sns
import pandas as pd
import re

cmap = sns.color_palette()
sns.set(style="ticks")

def func(x, a, b):
    return a * x + b

def time_string_to_seconds(time_string):
    hours = minutes = seconds = 0.0
    pattern = r'(?:(\d+\.?\d*)h)?\s*(?:(\d+\.?\d*)m)?\s*(?:(\d+\.?\d*)s)?'
    match = re.match(pattern, time_string.strip())
    if match:
        hour_str, minute_str, second_str = match.groups()
        if hour_str:
            hours = float(hour_str)
        if minute_str:
            minutes = float(minute_str)
        if second_str:
            seconds = float(second_str)
        total_seconds = hours * 3600 + minutes * 60 + seconds
        return total_seconds
    else:
        raise ValueError(f"Invalid time format: '{time_string}'")

def readtime(path, x, x1):
    t = []
    tw = []
    Nu = []
    Nr = []
    Nk = []
    Nq = []
    Nb = x * 2 + 4
    for i in x:
        with open(path + '/a' + str(i) + '/ht/band.out', 'r') as f:
            data = f.readlines()
            data = [x.split() for x in data]
        for j in data:
            if len(j) >= 4 and j[-1] == 'WALL':
                t.append(time_string_to_seconds(j[-2]))
            elif len(j) == 3 and ('rank' in j):
                Nu.append(int(j[2]))
            elif len(j) == 5 and j[0] == 'wavefunction':
                Nr.append(int(j[2]) * int(j[3]) * int(j[4]))
        with open(path + '/a' + str(i) + '/ht/scf.in', 'r') as f:
            data = f.readlines()
            data = [x.split() for x in data]
        Nk.append(np.prod(np.array(data[-1][:3], dtype='int')))
        with open(path + '/a' + str(i) + '/ht/band.in', 'r') as f:
            data = f.readlines()
            data = [x.split() for x in data]
        Nq.append(int(data[-2][3]))
    for i in x1:
        with open(path + '/a' + str(i) + '/wi/pw2wan.out', 'r') as f:
            data = f.readlines()
            data = [x.split() for x in data]
        for j in data:
            if len(j) >= 4 and j[-1] == 'WALL':
                t1 = time_string_to_seconds(j[-2])
        with open(path + '/a' + str(i) + '/wi/si8.wout', 'r') as f:
            data = f.readlines()
            data = [x.split() for x in data]
        for j in data:
            if len(j) == 5 and j[0] == 'Total':
                t2 = float(j[-2])
        tw.append(t1 + t2)
    t = np.array(t)
    tw = np.array(tw)
    tw = np.array(tw)
    return [t, tw]

fig,ax = plt.subplots(figsize=(3,4.8))

x2 = np.array([3, 4, 5, 6, 7, 8, 9, 10])
res = readtime('nk', x2, x2)
t2 = res[0]
t21 = res[1]
x2 = np.power(x2, 3)
p2, pcov = curve_fit(func, np.log(x2), np.log(t2))
p21, pcov = curve_fit(func, np.log(x2), np.log(t21))
df2 = pd.DataFrame(t2, index=x2)
df21 = pd.DataFrame(t21, index=x2)
fitdf2 = pd.DataFrame({'a': np.exp(func(np.log(x2), *p2))}, index=x2)
fitdf21 = pd.DataFrame({'a': np.exp(func(np.log(x2), *p21))}, index=x2)
sns.scatterplot(data=df2, markers='o', palette=(cmap[0],), legend=False)
sns.scatterplot(data=df21, markers='o', palette=(cmap[1],), legend=False)
sns.lineplot(data=fitdf2, legend=False, palette=(cmap[0],))
sns.lineplot(data=fitdf21, legend=False, palette=(cmap[1],))

plt.xlim(13, 1200)

plt.text(.65, .62, '$N_k^{0.96}$', transform=ax.transAxes, size=12, color=cmap[1])
plt.text(.7, .25, '$N_k^{0.62}$', transform=ax.transAxes, size=12, color=cmap[0])
l0, = ax.plot([], [], color=cmap[0], linewidth=1.5, marker='o', markersize=4.5)
l1, = ax.plot([], [], color=cmap[1], linewidth=1.5, marker='o', markersize=4.5)
legend = plt.legend((l0, l1), ('HT, 1 CPU core', 'WI, 1 CPU core'), loc='upper left', frameon=False)
ax.add_artist(legend)

plt.yscale("log")
plt.xscale("log")

ax.set_ylabel('Total time (s)')
ax.set_xlabel('$N_k$')
plt.savefig('Nk_scale.pdf', bbox_inches='tight')
plt.show()
