#!/usr/bin/env python
#2022/1/22 modify read_band
import numpy as np
import f90nml
import xml.etree.ElementTree as ET
import os
import sys
sys.path.append(os.getenv('HOME')+'/script')

hartree=27.211386245988
bohr=0.529177210903

def read_scf(path,filename=''):
    files = [f for f in os.listdir(path) if os.path.isfile(os.path.join(path, f))]
    if not filename:
        if 'scf.in' in files:
            filename='scf.in'
        else:
            names=[f for f in files if f[-2:]=='in']
            if len(names)==1:
                filename = names[0]
    nml=f90nml.read(path+'/'+filename)
    prefix = nml['control']['prefix']
    try:
        outdir = nml['control']['outdir']
    except:
        outdir = './'
    return(prefix,outdir)

def read_band(path):
    with open(path+'/band.in','r') as f:
        data=f.readlines()
    data=[x.split() for x in data]
    kpath=[]
    klabel=[]
    klen=[]
    for idx,l in enumerate(data):
        if len(l)==0: continue
        if l[0]=='K_POINTS':
            mode=l[1]
            num=int(data[idx+1][0])
            for i in data[idx+2:idx+2+num]:
                if (len(i)==0) or (i[0][0] in ('#','!')): continue
                assert len(i)==0 or len(i)>=3, 'K_POINTS error'
                kpath.append(i[:3])
                klen.append(int(float(i[3])))
                if len(i)>=5:
                    if i[4]=='G':
                        i[4]=r'$\Gamma$'
                    klabel.append(i[4])
                else:
                    klabel.append('')
            break
    assert mode in ('tpiba_b','crystal_b','crystal'), 'K_POINTS mode error'
    kpath=np.array(kpath,dtype='float')
    klen=np.array(klen,dtype='int')
    kinfo={'mode':mode,'kpath':kpath,'klen':klen,'klabel':klabel}
    return kinfo

def read_xml(path,prefix,outdir):
    cry=dict()
    root = ET.parse(path+'/'+outdir+'/'+prefix+'.xml').getroot()
    cell_obj = root.findall('./output/atomic_structure/cell/')
    cry['cell'] = np.array([i.text.split() for i in cell_obj],dtype='float')
    cry['funct'] = root.find('./output/dft/functional').text
    kpt_obj = root.findall('./output/band_structure/starting_k_points/k_point')
    cry['kpoints'] = np.array([i.text.split() for i in kpt_obj],dtype='float')
    cry['weight'] = np.array([i.attrib['weight'] for i in kpt_obj],dtype='float')
    ks_obj = root.findall('./output/band_structure/ks_energies')
    cry['ks_kpts'] = np.array([i[0].text.split() for i in ks_obj],dtype='float')
    cry['eigenvalues'] = np.array([i[2].text.split() for i in ks_obj],dtype='float')*hartree
    cry['fermi'] = float(root.find('./output/band_structure/fermi_energy').text)
    cry['nbnd'] = int(root.find('./output/band_structure/nbnd').text)
    cry['alat'] = float(root.findall('./output/atomic_structure')[0].attrib['alat'])
    cry['mass'] = {}
    for i in root.find('./output/atomic_species'):
        cry['mass'][i.attrib['name']]=float(i[0].text)
    pos_obj = root.find('./output/atomic_structure/atomic_positions')
    cry['pos'] = np.array([i.text.split() for i in pos_obj],dtype='float')
    cry['atomic_species'] = [i.attrib['name'] for i in pos_obj]
    return cry

def read_matdyn_modes(path):
    with open(path+'/matdyn.modes','r') as f:
        data=f.readlines()
    idx = -1
    nq=0
    total_atoms=0
    total_modes=0
    mode = []
    qpoints=[]
    freq = []
    for idx,line in enumerate(data):
        line=line.split()
        if len(line)==0: continue
        if line[0]=='q':
            nq+=1
            qpoints.append(line[2:5])
        if line[0]=='freq':
            total_modes+=1
            freq.append(line[7])
        if line[0]=='(':
            total_atoms+=1
            assert len(line)==8, 'Spaces may be lost in the file.'
            mode.append(line[1:6:2])
    nmodes=int(total_modes/nq)
    natoms=int(total_atoms/total_modes)
    assert total_modes==nq*nmodes, 'total number of modes cannot be divided by qpoints'
    assert total_atoms==nq*nmodes*natoms, 'total number of atoms cannot be divided by modes'
    res = dict()
    res['mode'] = np.array(mode,dtype='float').reshape([nq,nmodes,natoms,3])
    res['qpoints'] = np.array(qpoints,dtype='float')
    res['freq(cm-1)'] = np.array(freq, dtype='float').reshape([nq,nmodes])
    return res
